/*
                                  Langton
                                  -------
   
   Sencillo programa que utiliza el algoritmo de la hormiga de Langton y una
   implementacion configurable desde un fichero externo (langton.ini) para
   recrear un mundo virtual de hormigas. 
   
   Mas informacion en README.TXT
	   
	 - Email megastorm@mi.madritel.es
	 - IRC hispano: TheAzazel
     - www.megastormsystems.com  
*/

#define LANGTON_VER "Langton v1.0"

// ---Includes---
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>         
#include <string.h>
#include "sdl/sdl.h"

#include "langton.h" // Declaracion de la clase

// ---Prototipos definidos en PixelSDL.cpp---
void SDL_PutPixel(SDL_Surface *s, Sint16 x, Sint16 y, Uint32 color);
Uint32 SDL_GetPixel(SDL_Surface *s, Sint16 x, Sint16 y);

// ---Prototipos definidos en LoadIni.cpp---
Uint8 LoadIni(Uint8 *speed,cAntLangton **ants,Uint8 maxants);

// ---Variables globales---
SDL_Surface *screen;
Uint8 *keystate;

// ---Implementacion de la clase cAntLangton---
// Fija las coordenadas iniciales
void cAntLangton::Init(Uint16 x, Uint16 y, Uint8 vd) 
   { 
	   PosX=x; 
	   PosY=y; 
	   vectordirector=vd;
	   // Si ya existe screen, buscamos los colores
	   if(screen)
	   {		   
		   iColorRojo=SDL_MapRGB(screen->format,250,0,0);
		   iColorNegro=SDL_MapRGB(screen->format,0,0,0);
		   iColorVerde=SDL_MapRGB(screen->format,rand()%255,rand()%255,rand()%255);
	   }
   }

// Mueve la hormiga
Uint8 cAntLangton::Move(SDL_Surface *sAntWorld)
   {
      if((PosX>RESX-TAMX) || (PosX<TAMX)) return BAD;
      if((PosY>RESY-TAMY) || (PosY<TAMY)) return BAD;

      switch(vectordirector)
       {
         case 0:        // Izquierda
                PosX=PosX-TAMX;
                currentcolor=SDL_GetPixel(sAntWorld,PosX, PosY);				
                if(currentcolor==iColorRojo)
                 { 
					 SDL_PutPixel(sAntWorld,PosX, PosY, iColorVerde); 
					 vectordirector=3; 
					 return WELL;
				 }
				else
                 { 
					 SDL_PutPixel(sAntWorld,PosX, PosY, iColorRojo); 
					 vectordirector=2; 
					 return WELL;
				 }
                
                return BAD;
                break;

         case 1:        // Derecha
                PosX=PosX+TAMX;
                currentcolor=SDL_GetPixel(sAntWorld,PosX, PosY);
                if(currentcolor==iColorRojo)
                 { 
					 SDL_PutPixel(sAntWorld,PosX, PosY, iColorVerde); 
					 vectordirector=2; 
					 return WELL;
				 }
				else
                 { 
					 SDL_PutPixel(sAntWorld,PosX, PosY, iColorRojo); 
					 vectordirector=3; 
					 return WELL;
				 }                
                return BAD;
                break;

         case 2:        // Arriba
                PosY=PosY-TAMY;				
                currentcolor=SDL_GetPixel(sAntWorld,PosX, PosY);				
                if(currentcolor==iColorRojo)
                 { 
					 SDL_PutPixel(sAntWorld,PosX, PosY, iColorVerde); 
					 vectordirector=0; 
					 return WELL;
				 }
				else
                 { 
					 SDL_PutPixel(sAntWorld,PosX, PosY, iColorRojo); 
					 vectordirector=1; 
					 return WELL;
				 }                
                return BAD;
                break;

         case 3:        // Abajo
                PosY=PosY+TAMY;
                currentcolor=SDL_GetPixel(sAntWorld,PosX, PosY);
                if(currentcolor==iColorRojo)
                 { 
					 SDL_PutPixel(sAntWorld,PosX, PosY, iColorVerde); 
					 vectordirector=1; 
					 return WELL;
				 }
				else				
                 { 
					 SDL_PutPixel(sAntWorld,PosX, PosY, iColorRojo); 
					 vectordirector=0; 
					 return WELL;
				 }                
                return BAD;
                break;
        }
      return BAD;
   }

 Uint8 cAntLangton::iAlifeAnts=0;

// --------------------------------------------------------------------------
// --------------------------FUNCION PRINCIPAL-------------------------------
// --------------------------------------------------------------------------
int main( int argc, char* argv[] )
{

   SDL_Surface *sAntWorld;
   Uint8 flag=WELL,key=0;   
   Uint32 step=0,done=0,laststep=0,i=0;      
   clock_t tstart, tend;
   Uint8 iNumAnts;              // Numero de hormigas, de 1 a MAXANTS
   Uint8 iSpeed;                // 0: velocidad de CPU/VIDEO  1: resultado final  x: iteraciones por segundo
   cAntLangton *Ants[MAXANTS];  // Tabla estatica de punteros a las hormigas

   // Iniciamos sistema por defecto
   printf("-------------------------------------------------------------------------------\n");
   printf("----------------------La Hormiga de Langton por Roberto Prieto-----------------\n");
   printf("-----------------------------Portado a SDL el 10.02.2002-----------------------\n"); 
   printf("-------------------------------------------------------------------------------\n\n");
   iSpeed=0; iNumAnts=0;
   for(i=0;i<MAXANTS;i++) Ants[i]=NULL;

   // Iniciamos sistema de video
   SDL_Init(SDL_INIT_VIDEO);
   SDL_WM_SetCaption(LANGTON_VER,NULL);  
   SDL_WM_SetIcon(SDL_LoadBMP("icono.bmp"), NULL);
   screen=SDL_SetVideoMode(RESX,RESY,0,SDL_SWSURFACE);
   sAntWorld=SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_RLEACCEL,screen->w,screen->h,screen->format->BitsPerPixel,
		  screen->format->Rmask,screen->format->Gmask,screen->format->Bmask,screen->format->Amask);

   // Limpiamos superficies de video   
   SDL_FillRect(screen,NULL,SDL_MapRGB(screen->format,0,0,0));
   SDL_FillRect(sAntWorld,NULL,SDL_MapRGB(sAntWorld->format,0,0,0));
   srand(SDL_GetTicks());

   // Cargamos fichero externo 'langton.ini' y creamos hormigas
   iNumAnts=LoadIni(&iSpeed,Ants,MAXANTS);
   if(!iNumAnts)
   {
	   printf("  Error abriendo 'langton.ini', funcionamiento por defecto.\n");   
	   Ants[0]=new cAntLangton;
	   Ants[0]->Init(RESX/2,RESY/2,0);
   }
   
   // Activamos el modo de funcionamiento
   switch(iSpeed)
   {
   case 0: printf("  Iteraciones limitadas por velocidad CPU/VIDEO\n"); break;
   case 1: printf("  Muestra imagen 'output.bmp' final cuando las hormigas mueren\n"); break;
   default:
	   printf("  Multiplicacion de iteraciones x%d\n",iSpeed); break;
   }
   

   // Bucle principal de iteracion de hormiguitas
   tstart=clock();
   while(!done)
   {     
	   // Movemos a las hormigas y mueren
       for(i=0;i<MAXANTS;i++) 
	   {
		   // Comprobamos que sigua "viva"
		   if(Ants[i]) 
		   { 
			   flag=Ants[i]->Move(sAntWorld); 
			   // Se ha muerto?
			   if(flag==BAD) 
			   { delete Ants[i]; Ants[i]=NULL; }
		   }
	   }

	   // Si pulsamos ESC salimos	   
	   SDL_PumpEvents(); 
	   keystate=SDL_GetKeyState(NULL);
	   if(keystate[SDLK_ESCAPE]) done=1;

	   // Dependiendo del sistema..realizamos un volcado o no
	   step++;     	 
	   if(iSpeed!=1)
	   {
		   if(iSpeed==0)
		   {
               SDL_BlitSurface(sAntWorld,NULL,screen,NULL);
	           SDL_Flip(screen);
		   }
		   else
		   {
			   if(step>laststep+iSpeed)
			   {
				   laststep=step;
				   SDL_BlitSurface(sAntWorld,NULL,screen,NULL);
	               SDL_Flip(screen);
			   }
		   }
	   }
	   if(!cAntLangton::iAlifeAnts) done=1;
    }
   tend=clock();
   
   // Liberamos y salimos
   SDL_SaveBMP(sAntWorld,"output.bmp");
   SDL_FreeSurface(sAntWorld);
   SDL_Quit();
   
   // Informacion final
   printf("\n  Numero de Iteraciones: %d\n",step);
   printf("  Numero de hormigas totales/vivas: %d/%d\n", iNumAnts,cAntLangton::iAlifeAnts);
   printf("  Tiempo en ejecucion: %.2f seg\n",(float)(tend-tstart)/(float)CLOCKS_PER_SEC);
   printf("  %2.f iteraciones por segundo\n", (float)step/((float)(tend-tstart)/(float)CLOCKS_PER_SEC));
   
   // Matamos las hormigas que queden
   for(i=0;i<MAXANTS;i++) if(Ants[i]) delete Ants[i]; 
   return 1;
 }
// --------------------------------------------------------------------------