/*
                                  PixelSDL
                                  --------
    Rutinas para obtener y activar pixeles sobre superficies de SDL.
	Soporta 8,16,24 y 32 bits.
   
	 - Email megastorm@mi.madritel.es
	 - IRC hispano: TheAzazel
     - www.megastormsystems.com  
*/

// ---Includes---
#include "sdl/sdl.h"


// Dibuja un pixel
void SDL_PutPixel(SDL_Surface *s, Sint16 x, Sint16 y, Uint32 color)
   {
	   // Bloqueamos la superficie
	   if(SDL_MUSTLOCK(s)) 
	   {
		   if(SDL_LockSurface(s)<0) 
			{
		      printf("  [SDL_PutPixel()] Error: %s\n",SDL_GetError()); return;
			}
	   }
       
	   // Dibujamos el pixel
	   switch(s->format->BytesPerPixel)
	   {
	   // Modo 8bit
	   case 1:
		   {
			*((Uint8*)s->pixels + y * s->pitch +x) =color;
		    break;
		   }

	   // Modo 16bit
	   case 2:
		   {
		    *((Uint16*)s->pixels + y * s->pitch/2 +x) =color;
		    break;
		   }

	   //Modo 24bit
	   case 3:
		   {
		    Uint8 *pix=(Uint8*)s->pixels + y * s->pitch +x *3;
			*(pix+s->format->Rshift/8)=color>>s->format->Rshift;
			*(pix+s->format->Gshift/8)=color>>s->format->Gshift;
			*(pix+s->format->Bshift/8)=color>>s->format->Bshift;
			*(pix+s->format->Ashift/8)=color>>s->format->Ashift;
			break;
		   }
		   
	   // Modo 32bit
	   case 4:
		   {
			*((Uint32*)s->pixels + y * s->pitch/4 +x) =color;			
		    break;
		   }
	   }
       // Desbloqueamos la superficie
	   if(SDL_MUSTLOCK(s))
	   {
		   SDL_UnlockSurface(s);
	   }
   }
   


// Obtiene un pixel
Uint32 SDL_GetPixel(SDL_Surface *s, Sint16 x, Sint16 y)
   {
	   
	    Uint32 ret;
		
	    // Bloqueamos la superficie
	    if(SDL_MUSTLOCK(s)) 
		{
		   if(SDL_LockSurface(s)<0) 
			{
		      printf("  [SDL_GetPixel()] Error: %s\n",SDL_GetError()); return 0;
			}
		}

	   // Depende del modo grafico
	   switch(s->format->BytesPerPixel)
	   {
	   // Modo 8bit con paleta
	   case 1: 
		   {
			   ret=*((Uint8*)s->pixels + y * s->pitch + x);
		       break;
		   }
	   
	   // Modo 16bit
	   case 2: 
		   {
			   ret=*((Uint16*)s->pixels + y * s->pitch/2 + x);
		       break;
		   }
	   
	   // Modo 24bit (muy lento)
	   case 3: 
		   {
			   Uint8 *pix;
			   int shift;
			   Uint32 color;
			   
			   pix=(Uint8*)s->pixels + y * s->pitch + x * 3;
			   shift=s->format->Rshift;
			   color=*(pix+shift/8)<<shift;
			   shift=s->format->Gshift;
			   color|=*(pix+shift/8)<<shift;
			   shift=s->format->Bshift;
			   color|=*(pix+shift/8)<<shift;
			   shift=s->format->Ashift;
			   color|=*(pix+shift/8)<<shift;
			   ret=color;
			   break;
		   }

	   // Modo 32bit
	   case 4:
		   {
			   ret=*((Uint32*)s->pixels + y * s->pitch/4 + x);
			   break;
		   }
	   }

	   // Desbloqueamos la superficie
	    if(SDL_MUSTLOCK(s))
		{ 
		   SDL_UnlockSurface(s);
		}
	   return ret;
   }